<?php

namespace Drupal\micronode_block\Plugin\Block;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Drupal\node\Entity\Node;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a generic micronode block.
 *
 * @Block(
 *  id = "micronode_block",
 *  admin_label = @Translation("Micronode Block"),
 *  category = @Translation("Component"),
 *  deriver = "Drupal\micronode_block\Plugin\Derivative\MicronodeBlockDeriver"
 * )
 */
class MicronodeBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity display repository service.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected $entityDisplayRepository;

  /**
   * Constructs a new EntityBrowserBlock.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, EntityDisplayRepositoryInterface $entity_display_repository) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->entityDisplayRepository = $entity_display_repository;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_display.repository')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'nid' => NULL,
    ];
  }

  /**
   * Overrides \Drupal\Core\Block\BlockBase::blockForm().
   *
   * Adds body and description fields to the block configuration form.
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form['selection'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'entity-browser-block-form',
      ],
      '#attached' => [
        'library' => ['micronode_block/micronode_block'],
      ],
      // This element exists solely because Drupal.AjaxCommands.prototype.entity_browser_value_updated looks for this element.
      'target_id' => [
        '#type' => 'hidden',
        '#id' => 'entity-browser-block-form-target-id',
        '#attributes' => ['id' => 'entity-browser-block-form-target-id'],
        '#default_value' => NULL,
        '#ajax' => [
          'callback' => [self::class, 'updateCallback'],
          'wrapper' => 'entity-browser-block-form',
          'event' => 'entity_browser_value_updated',
        ],
      ],
    ];

    $form['selection']['entity_browser'] = [
      '#type' => 'entity_browser',
      '#entity_browser' => 'micronode_entity_browser',
      '#cardinality' => 1,
      '#process' => [
        [
          '\Drupal\entity_browser\Element\EntityBrowserElement',
          'processEntityBrowser',
        ],
        [self::class, 'processEntityBrowser'],
      ],
      // The default_value needs to look like ['node:1234' => $node] where $node is the loaded Node 1234.
      '#default_value' => $this->configuration['nid'] ? ['node:' . $this->configuration['nid'] => $this->entityTypeManager->getStorage('node')->load($this->configuration['nid'])] : [],
      '#widget_context' => [
        'target_bundles' => [
          $this->getDerivativeId() => $this->getDerivativeId(),
        ],
        'bundle' => $this->getDerivativeId(),
        'bundle_label' => $this->entityTypeManager->getStorage('node_type')->load($this->getDerivativeId())->label(),
      ],
    ];

    // We put this code low so that the form field updates before the preview.
    // But we set the weight low so it renders above.
    $form['selection']['preview'] = [
      '#type' => 'container',
      '#process' => [
        [self::class, 'processPreview'],
      ],
      '#weight' => -100,
    ];

    return $form;
  }

  /**
   * Render API callback: Processes the entity browser element.
   */
  public static function processEntityBrowser(&$element, FormStateInterface $form_state, &$complete_form) {
    $element['entity_ids']['#ajax'] = [
      'callback' => [self::class, 'updateCallback'],
      'wrapper' => 'entity-browser-block-form',
      'event' => 'entity_browser_value_updated',
    ];
    $element['entity_ids']['#default_value'] = implode(' ', array_keys($element['#default_value']));
    return $element;
  }

  /**
   * AJAX callback: Re-renders the Entity Browser button/table.
   */
  public static function updateCallback(array &$form, FormStateInterface $form_state) {
    $trigger = $form_state->getTriggeringElement();
    if (isset($trigger['#op']) && $trigger['#op'] === 'remove') {
      $parents = array_slice($trigger['#array_parents'], 0, -4);
      $selection = NestedArray::getValue($form, $parents);
      unset($selection['preview']);
      $selection['entity_browser']['entity_ids']['#value'] = NULL;
    }
    else {
      $parents = array_slice($trigger['#array_parents'], 0, -2);
      $selection = NestedArray::getValue($form, $parents);
    }
    return $selection;
  }

  /**
   * Render API callback: Processes the preview element.
   */
  public static function processPreview(&$element, FormStateInterface $form_state, &$complete_form) {
    $parents = array_slice($element['#array_parents'], -3, 2);
    $entity_ids = $form_state->getValue(array_merge($parents, ['entity_browser', 'entity_ids']), '');
    $nid = explode(":", $entity_ids)[1] ?? NULL;
    if (empty($nid)) {
      return [];
    }
    $node = Node::load($nid);
    $bundle_label = \Drupal::service('entity_type.manager')->getStorage('node_type')->load($node->getType())->label();
    $view_buider = \Drupal::service('entity_type.manager')->getViewBuilder('node');
    $element[$nid] = [
      'header' => [
        '#type' => 'html_tag',
        '#tag' => 'h2',
        '#value' => "Currently Selected $bundle_label",
      ],
      'preview' => [
        '#type' => 'details',
        '#open' => TRUE,
        '#title' => $node->access('view label') ? t('@type : @label', ['@type' => $bundle_label, '@label' => $node->label()]) : t('You do not have permission to see this node'),
        'render' => $node->access('view') ? $view_buider->view($node, 'micronode_preview') : ['#markup' => 'You do not have permission to see this node'],
      ],
      'operations' => [
        'edit_button' => [
          '#type' => 'submit',
          '#value' => "Edit Selected $bundle_label",
          '#name' => 'edit_button_' . $nid,
          '#ajax' => [
            'url' => Url::fromRoute(
              'entity_browser.edit_form', [
                'entity_type' => 'node',
                'entity' => $node->id(),
              ]),
            'options' => [
              'query' => [
                'details_id' => 'entity-browser-block-form',
              ],
            ],
          ],
          '#attributes' => [
            'class' => ['edit-button'],
          ],
          '#access' => $node->access('update'),
        ],
        'remove' => [
          '#type' => 'button',
          '#value' => "Remove $bundle_label from Block",
          '#op' => 'remove',
          '#name' => 'remove_' . $nid,
          '#ajax' => [
            'callback' => [self::class, 'updateCallback'],
            'wrapper' => 'entity-browser-block-form',
          ],
        ],
      ],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $selection = $form_state->getValue('selection');
    if (!empty($selection['entity_browser']['entity_ids'])) {
      // The value of $selection['entity_browser']['entity_ids'] looks like ['node:1234' => $node] where $node is the loaded Node 1234.
      // We just want to store the nid in configuration.
      $this->configuration['nid'] = explode(":", $selection['entity_browser']['entity_ids'])[1] ?? NULL;
      $node_type = $this->entityTypeManager->getStorage('node_type')->load($this->getDerivativeId());
      $node = $this->entityTypeManager->getStorage('node')->load($this->configuration['nid']);
      $this->configuration['label'] = $node_type->label() . ': ' . $node->label();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function blockValidate($form, FormStateInterface $form_state) {
    // Only validate during real submission.
    $trigger = $form_state->getTriggeringElement();
    if ($trigger['#name'] === 'op') {
      $selection = $form_state->getValue('selection');
      if (empty($selection['entity_browser']['entity_ids'])) {
        $form_state->setErrorByName('entity_browser', $this->t('You must add Microcontent to this Block'));
      }
      else {
        $nid = explode(":", $selection['entity_browser']['entity_ids'])[1] ?? NULL;
        if ($nid && $node = Node::load($nid)) {
          if ($node->getType() !== $this->getDerivativeId()) {
            $form_state->setErrorByName('entity_browser', $this->t('The selected Microcontent must be of the type @type. The selected content has the type @bad.', ['@type' => $this->getDerivativeId(), '@bad' => $node->getType()]));
          }
        }
        else {
          $form_state->setErrorByName('entity_browser', $this->t('Something is seriously wrong with the selected microcontent. Try removing it and starting over.'));
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    if ($this->configuration['nid']) {
      $entity = $this->entityTypeManager->getStorage('node')->load($this->configuration['nid']);
      if ($entity) {
        $access = $entity->access('view', NULL, TRUE);
        if ($access->isAllowed()) {
          $render = $this->entityTypeManager->getViewBuilder('node')->view($entity, 'default');
        }
        else {
          $render = [];
        }
        // We need to add the cacheability of access for security reasons.
        CacheableMetadata::createFromObject($access)->applyTo($render);
        $build = $render;
      }
    }
    return $build;
  }

}
