<?php

declare(strict_types=1);

namespace Drupal\Tests\micronode_block\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\node\Entity\Node;

/**
 * Tests all the form and browser stuff.
 *
 * @group micronode_block
 */
class MicronodeBlockFormTest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'micronode_block_test',
    'block',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Tests that Micronode Block installs and its deriver works.
   **/
  public function testMicronodeBlockForm() {
    $this->drupalLogin($this->createUser([], 'radmin', TRUE));
    $this->drupalGet('/admin/structure/block/add/micronode_block:two');
    $page = $this->getSession()->getPage();

    // The modal should open automatically since the block is empty.
    $modal_title_selector = '.ui-dialog-title[data-once="micronode_block_rewrite_modal_title"]';
    $this->assertNotNull($this->assertSession()->waitForElement('css', $modal_title_selector));
    $this->assertSession()->elementTextEquals('css', $modal_title_selector, 'Add Two');

    // Close the modal.
    $page->pressButton('Close');

    // We should not be able to save the block.
    $this->assertSession()->elementAttributeExists('css', 'input[value="Save block"]', 'disabled');

    // Let's re-open the modal.
    $page->pressButton('Add Two');
    $modal_title_selector = '.ui-dialog-title[data-once="micronode_block_rewrite_modal_title"]';
    $this->assertNotNull($this->assertSession()->waitForElement('css', $modal_title_selector));
    $this->assertSession()->elementTextEquals('css', $modal_title_selector, 'Add Two');

    // Let's go into that form, which is an iframe.
    $this->getSession()->switchToIFrame('entity_browser_iframe_micronode_entity_browser');
    $iframe = $this->getSession()->getPage();

    // Tabs exist.
    $this->assertTrue($iframe->hasLink('New Two'));
    $this->assertTrue($iframe->hasLink('Existing Two'));
    $this->assertTrue($iframe->hasButton('Save Two'));

    // Make a new node.
    $title_field = $iframe->findField('Title');
    $title_field->setValue('My new two');
    $iframe->pressButton('Save Two');

    // Modal should be closing. Go back to main page.
    $this->getSession()->switchToIFrame();

    // Check on the preview.
    $this->assertNotNull($this->assertSession()->waitForElement('css', '[data-drupal-selector="edit-settings-selection-preview"]'));
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"]', 'Currently Selected Two');
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"] summary', 'Two : My new two');
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"] article h2', 'My new two');

    // Save it after setting the region.
    $region_field = $page->findField('Region');
    $region_field->setValue('content');
    $this->assertSession()->elementAttributeNotExists('css', 'input[value="Save block"]', 'disabled');
    $page->pressButton('Save block');
    $this->assertSession()->statusMessageContains('The block configuration has been saved.');

    // The block should be on whatever is the front page.
    $this->drupalGet('<front>');
    $this->assertSession()->elementTextContains('css', '#block-two', 'My new two');

    // Inspect the new node directly.
    $this->assertSame('My new two', Node::load(1)->label());

    // Edit the block.
    $this->drupalGet('/admin/structure/block/manage/two');
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"]', 'Currently Selected Two');
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"] summary', 'Two : My new two');
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"] article h2', 'My new two');
    $page->hasButton('Edit Selected Two');
    $page->hasButton('Remove Two from Block');

    // Edit the selected node. This is quite different from creating the
    // node because it does not happen in an iframe or use entity_browser
    // directly.
    $page->pressButton('Edit Selected Two');
    $this->assertNotNull($this->assertSession()->waitForElement('css', 'form.micronode-block-edit-form'));
    $title_field = $page->find('css', 'form.micronode-block-edit-form [name="title[0][value]"]');
    $title_field->setValue('My edited two');
    $page->find('css', '.ui-dialog-buttonpane button')->click();

    // Check on the preview.
    $this->assertNotNull($this->assertSession()->waitForElement('css', '[data-drupal-selector="edit-settings-selection-preview"]'));
    $this->assertSession()->waitForText('Two : My edited two');
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"]', 'Currently Selected Two');
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"] summary', 'Two : My edited two');
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"] article h2', 'My edited two');

    // Save it.
    $this->assertSession()->elementAttributeNotExists('css', 'input[value="Save block"]', 'disabled');
    $page->pressButton('Save block');
    $this->assertSession()->statusMessageContains('The block configuration has been saved.');
    // We shouldn't have to clear cache. What!?
    // @todo Figure out why cache is wonky.
    drupal_flush_all_caches();
    $this->assertSame('My edited two', Node::load(1)->label());
    $this->drupalGet('<front>');
    $this->assertSession()->elementTextContains('css', '#block-two', 'My edited two');

    // Try removing the node and then using the existing node browser.
    Node::create(['type' => 'two', 'title' => 'My programmatic two'])->save();
    Node::create(['type' => 'three', 'title' => 'My programmatic three'])->save();
    $this->drupalGet('/admin/structure/block/manage/two');
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"]', 'Currently Selected Two');
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"] summary', 'Two : My edited two');
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"] article h2', 'My edited two');
    $page->hasButton('Edit Selected Two');
    $page->hasButton('Remove Two from Block');
    $page->pressButton('Remove Two from Block');
    // That should cause the modal to open.
    // The modal should open automatically since the block is empty.
    $modal_title_selector = '.ui-dialog-title[data-once="micronode_block_rewrite_modal_title"]';
    $this->assertNotNull($this->assertSession()->waitForElement('css', $modal_title_selector));
    $this->assertSession()->elementTextEquals('css', $modal_title_selector, 'Add Two');
    $this->getSession()->switchToIFrame('entity_browser_iframe_micronode_entity_browser');
    $iframe = $this->getSession()->getPage();
    $this->assertTrue($iframe->hasLink('New Two'));
    $this->assertTrue($iframe->hasLink('Existing Two'));
    // Close the modal.
    $this->getSession()->switchToIFrame();
    $page->pressButton('Close');
    // We should not be able to save the block.
    $this->assertSession()->elementAttributeExists('css', 'input[value="Save block"]', 'disabled');
    $this->assertSession()->pageTextNotContains('Currently Selected Two');
    // Let's re-open the modal.
    $page->pressButton('Add Two');
    $this->assertNotNull($this->assertSession()->waitForElement('css', $modal_title_selector));
    // Go into the existing node tab.
    $this->getSession()->switchToIFrame('entity_browser_iframe_micronode_entity_browser');
    $iframe->clickLink('Existing Two');
    $this->assertSession()->elementTextContains('css', '.views-row', 'My edited two');
    $this->assertSession()->elementTextContains('css', '.views-row', 'My programmatic two');
    $this->assertSession()->elementTextNotContains('css', '.views-row', 'My programmatic three');
    $iframe->find('css', '[value="node:2"]')->click();
    $iframe->pressButton('Select Two');

    // Check on the preview.
    $this->getSession()->switchToIFrame();
    $this->assertSession()->waitForText('Currently Selected Recipe');
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"]', 'Currently Selected Two');
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"] summary', 'Two : My programmatic two');
    $this->assertSession()->elementTextContains('css', '[data-drupal-selector="edit-settings-selection-preview"] article h2', 'My programmatic two');
    // Save it.
    $this->assertSession()->elementAttributeNotExists('css', 'input[value="Save block"]', 'disabled');
    $page->pressButton('Save block');
    $this->assertSession()->statusMessageContains('The block configuration has been saved.');
    $this->drupalGet('<front>');
    $this->assertSession()->elementTextContains('css', '#block-two', 'My programmatic two');
  }

}
